#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/5/9 17:06
Desc: 新浪行业-板块行情
http://finance.sina.com.cn/stock/sl/
"""

import json
import math

import pandas as pd
import requests

from akshare.utils import demjson
from tqdm import tqdm


def stock_sector_spot(indicator: str = "新浪行业") -> pd.DataFrame:
    """
    新浪行业-板块行情
    http://finance.sina.com.cn/stock/sl/
    :param indicator: choice of {"新浪行业", "启明星行业", "概念", "地域", "行业"}
    :type indicator: str
    :return: 指定 indicator 的数据
    :rtype: pandas.DataFrame
    """
    if indicator == "新浪行业":
        url = "http://vip.stock.finance.sina.com.cn/q/view/newSinaHy.php"
        r = requests.get(url)
    if indicator == "启明星行业":
        url = "http://biz.finance.sina.com.cn/hq/qmxIndustryHq.php"
        r = requests.get(url)
        r.encoding = "gb2312"
    if indicator == "概念":
        url = "http://money.finance.sina.com.cn/q/view/newFLJK.php"
        params = {"param": "class"}
        r = requests.get(url, params=params)
    if indicator == "地域":
        url = "http://money.finance.sina.com.cn/q/view/newFLJK.php"
        params = {"param": "area"}
        r = requests.get(url, params=params)
    if indicator == "行业":
        url = "http://money.finance.sina.com.cn/q/view/newFLJK.php"
        params = {"param": "industry"}
        r = requests.get(url, params=params)
    text_data = r.text
    json_data = json.loads(text_data[text_data.find("{") :])
    temp_df = pd.DataFrame([value.split(",") for key, value in json_data.items()])
    temp_df.columns = [
        "label",
        "板块",
        "公司家数",
        "平均价格",
        "涨跌额",
        "涨跌幅",
        "总成交量",
        "总成交额",
        "股票代码",
        "个股-涨跌幅",
        "个股-当前价",
        "个股-涨跌额",
        "股票名称",
    ]
    temp_df["公司家数"] = pd.to_numeric(temp_df["公司家数"])
    temp_df["平均价格"] = pd.to_numeric(temp_df["平均价格"])
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"])
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"])
    temp_df["总成交量"] = pd.to_numeric(temp_df["总成交量"])
    temp_df["总成交额"] = pd.to_numeric(temp_df["总成交额"])
    temp_df["个股-涨跌幅"] = pd.to_numeric(temp_df["个股-涨跌幅"])
    temp_df["个股-当前价"] = pd.to_numeric(temp_df["个股-当前价"])
    temp_df["个股-涨跌额"] = pd.to_numeric(temp_df["个股-涨跌额"])
    return temp_df


def stock_sector_detail(sector: str = "gn_gfgn") -> pd.DataFrame:
    """
    新浪行业-板块行情-成份详情
    http://finance.sina.com.cn/stock/sl/#area_1
    :param sector: stock_sector_spot 返回的 label 值, choice of {"新浪行业", "概念", "地域", "行业"}; "启明星行业" 无详情
    :type sector: str
    :return: 指定 sector 的板块详情
    :rtype: pandas.DataFrame
    """
    url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCount"
    params = {"node": sector}
    r = requests.get(url, params=params)
    total_num = int(r.json())
    total_page_num = math.ceil(int(total_num) / 80)
    big_df = pd.DataFrame()
    url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData"
    for page in tqdm(range(1, total_page_num + 1), leave=True):
        params = {
            "page": str(page),
            "num": "80",
            "sort": "symbol",
            "asc": "1",
            "node": sector,
            "symbol": "",
            "_s_r_a": "page",
        }
        r = requests.get(url, params=params)
        data_text = r.text
        data_json = demjson.decode(data_text)
        temp_df = pd.DataFrame(data_json)
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["trade"] = pd.to_numeric(big_df["trade"], errors="coerce")
    big_df["pricechange"] = pd.to_numeric(big_df["pricechange"], errors="coerce")
    big_df["changepercent"] = pd.to_numeric(big_df["changepercent"], errors="coerce")
    big_df["buy"] = pd.to_numeric(big_df["buy"], errors="coerce")
    big_df["sell"] = pd.to_numeric(big_df["sell"], errors="coerce")
    big_df["settlement"] = pd.to_numeric(big_df["settlement"], errors="coerce")
    big_df["open"] = pd.to_numeric(big_df["open"], errors="coerce")
    big_df["high"] = pd.to_numeric(big_df["high"], errors="coerce")
    big_df["low"] = pd.to_numeric(big_df["low"], errors="coerce")
    big_df["volume"] = pd.to_numeric(big_df["volume"], errors="coerce")
    big_df["amount"] = pd.to_numeric(big_df["amount"], errors="coerce")
    big_df["per"] = pd.to_numeric(big_df["per"], errors="coerce")
    big_df["pb"] = pd.to_numeric(big_df["pb"], errors="coerce")
    big_df["mktcap"] = pd.to_numeric(big_df["mktcap"], errors="coerce")
    big_df["nmc"] = pd.to_numeric(big_df["nmc"], errors="coerce")
    big_df["turnoverratio"] = pd.to_numeric(big_df["turnoverratio"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_industry_sina_df = stock_sector_spot(indicator="新浪行业")
    print(stock_industry_sina_df)

    stock_industry_con_df = stock_sector_spot(indicator="概念")
    print(stock_industry_con_df)

    stock_industry_area_df = stock_sector_spot(indicator="地域")
    print(stock_industry_area_df)

    stock_industry_ind_df = stock_sector_spot(indicator="行业")
    print(stock_industry_ind_df)

    stock_industry_star_df = stock_sector_spot(indicator="启明星行业")
    print(stock_industry_star_df)

    stock_sector_detail_df = stock_sector_detail(sector="hangye_ZC27")
    print(stock_sector_detail_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/5/27 15:30
Desc: 股票基本信息
"""

import json
import warnings
from functools import lru_cache
from io import BytesIO
from io import StringIO

import pandas as pd
import requests
from akshare.utils.tqdm import get_tqdm


@lru_cache()
def stock_info_sz_name_code(symbol: str = "A股列表") -> pd.DataFrame:
    """
    深圳证券交易所-股票列表
    https://www.szse.cn/market/product/stock/list/index.html
    :param symbol: choice of {"A股列表", "B股列表", "CDR列表", "AB股列表"}
    :type symbol: str
    :return: 指定 indicator 的数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.szse.cn/api/report/ShowReport"
    indicator_map = {
        "A股列表": "tab1",
        "B股列表": "tab2",
        "CDR列表": "tab3",
        "AB股列表": "tab4",
    }
    params = {
        "SHOWTYPE": "xlsx",
        "CATALOGID": "1110",
        "TABKEY": indicator_map[symbol],
        "random": "0.6935816432433362",
    }
    r = requests.get(url, params=params, timeout=15)
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("always")
        temp_df = pd.read_excel(BytesIO(r.content))
    if len(temp_df) > 10:
        if symbol == "A股列表":
            temp_df["A股代码"] = (
                temp_df["A股代码"]
                .astype(str)
                .str.split(".", expand=True)
                .iloc[:, 0]
                .str.zfill(6)
                .str.replace("000nan", "")
            )
            temp_df = temp_df[
                [
                    "板块",
                    "A股代码",
                    "A股简称",
                    "A股上市日期",
                    "A股总股本",
                    "A股流通股本",
                    "所属行业",
                ]
            ]
        elif symbol == "B股列表":
            temp_df["B股代码"] = (
                temp_df["B股代码"]
                .astype(str)
                .str.split(".", expand=True)
                .iloc[:, 0]
                .str.zfill(6)
                .str.replace("000nan", "")
            )
            temp_df = temp_df[
                [
                    "板块",
                    "B股代码",
                    "B股简称",
                    "B股上市日期",
                    "B股总股本",
                    "B股流通股本",
                    "所属行业",
                ]
            ]
        elif symbol == "AB股列表":
            temp_df["A股代码"] = (
                temp_df["A股代码"]
                .astype(str)
                .str.split(".", expand=True)
                .iloc[:, 0]
                .str.zfill(6)
                .str.replace("000nan", "")
            )
            temp_df["B股代码"] = (
                temp_df["B股代码"]
                .astype(str)
                .str.split(".", expand=True)
                .iloc[:, 0]
                .str.zfill(6)
                .str.replace("000nan", "")
            )
            temp_df = temp_df[
                [
                    "板块",
                    "A股代码",
                    "A股简称",
                    "A股上市日期",
                    "B股代码",
                    "B股简称",
                    "B股上市日期",
                    "所属行业",
                ]
            ]
        return temp_df
    else:
        return temp_df


@lru_cache()
def stock_info_sh_name_code(symbol: str = "主板A股") -> pd.DataFrame:
    """
    上海证券交易所-股票列表
    https://www.sse.com.cn/assortment/stock/list/share/
    :param symbol: choice of {"主板A股": "1", "主板B股": "2", "科创板": "8"}
    :type symbol: str
    :return: 指定 indicator 的数据
    :rtype: pandas.DataFrame
    """
    indicator_map = {"主板A股": "1", "主板B股": "2", "科创板": "8"}
    url = "https://query.sse.com.cn/sseQuery/commonQuery.do"
    headers = {
        "Host": "query.sse.com.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.sse.com.cn/assortment/stock/list/share/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
    }
    params = {
        "STOCK_TYPE": indicator_map[symbol],
        "REG_PROVINCE": "",
        "CSRC_CODE": "",
        "STOCK_CODE": "",
        "sqlId": "COMMON_SSE_CP_GPJCTPZ_GPLB_GP_L",
        "COMPANY_STATUS": "2,4,5,7,8",
        "type": "inParams",
        "isPagination": "true",
        "pageHelp.cacheSize": "1",
        "pageHelp.beginPage": "1",
        "pageHelp.pageSize": "10000",
        "pageHelp.pageNo": "1",
        "pageHelp.endPage": "1",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    col_stock_code = "B_STOCK_CODE" if symbol == "主板B股" else "A_STOCK_CODE"
    temp_df.rename(
        columns={
            col_stock_code: "证券代码",
            "COMPANY_ABBR": "证券简称",
            "FULL_NAME": "公司全称",
            "LIST_DATE": "上市日期",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "公司全称",
            "上市日期",
        ]
    ]
    temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"], errors="coerce").dt.date
    return temp_df


@lru_cache()
def stock_info_bj_name_code() -> pd.DataFrame:
    """
    北京证券交易所-股票列表
    https://www.bse.cn/nq/listedcompany.html
    :return: 股票列表
    :rtype: pandas.DataFrame
    """
    url = "https://www.bse.cn/nqxxController/nqxxCnzq.do"
    payload = {
        "page": "0",
        "typejb": "T",
        "xxfcbj[]": "2",
        "xxzqdm": "",
        "sortfield": "xxzqdm",
        "sorttype": "asc",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/110.0.0.0 Safari/537.36"
    }
    r = requests.post(url, data=payload, headers=headers)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("[") : -1])
    total_page = data_json[0]["totalPages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(total_page), leave=False):
        payload.update({"page": page})
        r = requests.post(url, data=payload, headers=headers)
        data_text = r.text
        data_json = json.loads(data_text[data_text.find("[") : -1])
        temp_df = data_json[0]["content"]
        temp_df = pd.DataFrame(temp_df)
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "上市日期",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "流通股本",
        "-",
        "-",
        "-",
        "-",
        "-",
        "所属行业",
        "-",
        "-",
        "-",
        "-",
        "报告日期",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "地区",
        "-",
        "-",
        "-",
        "-",
        "-",
        "券商",
        "总股本",
        "-",
        "证券代码",
        "-",
        "证券简称",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "证券代码",
            "证券简称",
            "总股本",
            "流通股本",
            "上市日期",
            "所属行业",
            "地区",
            "报告日期",
        ]
    ]
    big_df["报告日期"] = pd.to_datetime(big_df["报告日期"], errors="coerce").dt.date
    big_df["上市日期"] = pd.to_datetime(big_df["上市日期"], errors="coerce").dt.date
    return big_df


def stock_info_sh_delist(symbol: str = "全部") -> pd.DataFrame:
    """
    上海证券交易所-终止上市公司
    https://www.sse.com.cn/assortment/stock/list/delisting/
    :param symbol: choice of {"全部", "沪市", "科创板"}
    :type symbol: str
    :return: 终止上市公司
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部": "1,2,8",
        "沪市": "1,2",
        "科创板": "8",
    }
    url = "https://query.sse.com.cn/commonQuery.do"
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "query.sse.com.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/97.0.4692.71 Safari/537.36",
    }
    params = {
        "sqlId": "COMMON_SSE_CP_GPJCTPZ_GPLB_GP_L",
        "isPagination": "true",
        "STOCK_CODE": "",
        "CSRC_CODE": "",
        "REG_PROVINCE": "",
        "STOCK_TYPE": symbol_map[symbol],
        "COMPANY_STATUS": "3",
        "type": "inParams",
        "pageHelp.cacheSize": "1",
        "pageHelp.beginPage": "1",
        "pageHelp.pageSize": "500",
        "pageHelp.pageNo": "1",
        "pageHelp.endPage": "1",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    temp_df.rename(
        columns={
            "COMPANY_ABBR": "公司简称",
            "DELIST_DATE": "暂停上市日期",
            "LIST_DATE": "上市日期",
            "COMPANY_CODE": "公司代码",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "公司代码",
            "公司简称",
            "上市日期",
            "暂停上市日期",
        ]
    ]
    temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"], errors="coerce").dt.date
    temp_df["暂停上市日期"] = pd.to_datetime(
        temp_df["暂停上市日期"], errors="coerce"
    ).dt.date
    return temp_df


def stock_info_sz_delist(symbol: str = "终止上市公司") -> pd.DataFrame:
    """
    深证证券交易所-暂停上市公司-终止上市公司
    https://www.szse.cn/market/stock/suspend/index.html
    :param symbol: choice of {"暂停上市公司", "终止上市公司"}
    :type symbol: str
    :return: 暂停上市公司 or 终止上市公司 的数据
    :rtype: pandas.DataFrame
    """
    indicator_map = {"暂停上市公司": "tab1", "终止上市公司": "tab2"}
    url = "https://www.szse.cn/api/report/ShowReport"
    params = {
        "SHOWTYPE": "xlsx",
        "CATALOGID": "1793_ssgs",
        "TABKEY": indicator_map[symbol],
        "random": "0.6935816432433362",
    }
    r = requests.get(url, params=params)
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("always")
        temp_df = pd.read_excel(BytesIO(r.content))
        if temp_df.empty:
            return pd.DataFrame()
        temp_df["证券代码"] = temp_df["证券代码"].astype("str").str.zfill(6)
        temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"]).dt.date
        temp_df["终止上市日期"] = pd.to_datetime(temp_df["终止上市日期"]).dt.date
        return temp_df


def stock_info_sz_change_name(symbol: str = "全称变更") -> pd.DataFrame:
    """
    深证证券交易所-市场数据-股票数据-名称变更
    https://www.szse.cn/www/market/stock/changename/index.html
    :param symbol: choice of {"全称变更": "tab1", "简称变更": "tab2"}
    :type symbol: str
    :return: 名称变更数据
    :rtype: pandas.DataFrame
    """
    indicator_map = {"全称变更": "tab1", "简称变更": "tab2"}
    url = "https://www.szse.cn/api/report/ShowReport"
    params = {
        "SHOWTYPE": "xlsx",
        "CATALOGID": "SSGSGMXX",
        "TABKEY": indicator_map[symbol],
        "random": "0.6935816432433362",
    }
    r = requests.get(url, params=params)
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("always")
        temp_df = pd.read_excel(BytesIO(r.content))
        temp_df["证券代码"] = temp_df["证券代码"].astype("str").str.zfill(6)
        temp_df["变更日期"] = pd.to_datetime(temp_df["变更日期"]).dt.date
        temp_df.sort_values(["变更日期"], inplace=True, ignore_index=True)
        return temp_df


def stock_info_change_name(symbol: str = "000503") -> pd.DataFrame:
    """
    新浪财经-股票曾用名
    https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_CorpInfo/stockid/300378.phtml
    :param symbol: 股票代码
    :type symbol: str
    :return: 股票曾用名
    :rtype: list
    """
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_CorpInfo/stockid/{symbol}.phtml"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[3].iloc[:, :2]
    temp_df.dropna(inplace=True)
    temp_df.columns = ["item", "value"]
    temp_df["item"] = temp_df["item"].str.split("：", expand=True)[0]
    try:
        name_list = (
            temp_df[temp_df["item"] == "证券简称更名历史"].value.tolist()[0].split(" ")
        )
        big_df = pd.DataFrame(name_list)
        big_df.reset_index(inplace=True)
        big_df["index"] = big_df.index + 1
        big_df.columns = ["index", "name"]
        return big_df
    except IndexError:
        return pd.DataFrame()


@lru_cache()
def stock_info_a_code_name() -> pd.DataFrame:
    """
    沪深京 A 股列表
    :return: 沪深京 A 股数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    stock_sh = stock_info_sh_name_code(symbol="主板A股")
    stock_sh = stock_sh[["证券代码", "证券简称"]]

    stock_sz = stock_info_sz_name_code(symbol="A股列表")
    stock_sz["A股代码"] = stock_sz["A股代码"].astype(str).str.zfill(6)
    big_df = pd.concat(
        objs=[big_df, stock_sz[["A股代码", "A股简称"]]], ignore_index=True
    )
    big_df.columns = ["证券代码", "证券简称"]

    stock_kcb = stock_info_sh_name_code(symbol="科创板")
    stock_kcb = stock_kcb[["证券代码", "证券简称"]]

    stock_bse = stock_info_bj_name_code()
    stock_bse = stock_bse[["证券代码", "证券简称"]]
    stock_bse.columns = ["证券代码", "证券简称"]

    big_df = pd.concat(objs=[big_df, stock_sh], ignore_index=True)
    big_df = pd.concat(objs=[big_df, stock_kcb], ignore_index=True)
    big_df = pd.concat(objs=[big_df, stock_bse], ignore_index=True)
    big_df.columns = ["code", "name"]
    return big_df


if __name__ == "__main__":
    stock_info_sh_name_code_df = stock_info_sh_name_code(symbol="主板A股")
    print(stock_info_sh_name_code_df)

    stock_info_sh_name_code_df = stock_info_sh_name_code(symbol="主板B股")
    print(stock_info_sh_name_code_df)

    stock_info_sh_name_code_df = stock_info_sh_name_code(symbol="科创板")
    print(stock_info_sh_name_code_df)

    stock_info_sz_name_code_df = stock_info_sz_name_code(symbol="A股列表")
    print(stock_info_sz_name_code_df)

    stock_info_sz_df = stock_info_sz_name_code(symbol="B股列表")
    print(stock_info_sz_df)

    stock_info_sz_df = stock_info_sz_name_code(symbol="AB股列表")
    print(stock_info_sz_df)

    stock_info_sz_df = stock_info_sz_name_code(symbol="CDR列表")
    print(stock_info_sz_df)

    stock_info_sh_delist_df = stock_info_sh_delist()
    print(stock_info_sh_delist_df)

    stock_info_sz_change_name_df = stock_info_sz_change_name(symbol="全称变更")
    print(stock_info_sz_change_name_df)

    stock_info_change_name_df = stock_info_change_name(symbol="000503")
    print(stock_info_change_name_df)

    stock_info_a_code_name_df = stock_info_a_code_name()
    print(stock_info_a_code_name_df)

    stock_info_bj_name_code_df = stock_info_bj_name_code()
    print(stock_info_bj_name_code_df)

    stock_info_sz_delist_df = stock_info_sz_delist(symbol="终止上市公司")
    print(stock_info_sz_delist_df)

    stock_info_sz_delist_df = stock_info_sz_delist(symbol="暂停上市公司")
    print(stock_info_sz_delist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/18 16:25
Desc: 新加坡交易所-衍生品-历史数据-历史结算价格
https://www.sgx.com/zh-hans/research-education/derivatives
https://links.sgx.com/1.0.0/derivatives-daily/5888/FUTURE.zip
"""

import zipfile
from io import BytesIO
from io import StringIO

import pandas as pd
import requests


def __fetch_ftse_index_futu(date: str = "20231108") -> int:
    """
    新加坡交易所-日历计算
    https://wap.eastmoney.com/quote/stock/100.STI.html
    :param date: 交易日
    :type date: str
    :return: 日期计算结果
    :rtype: int
    """
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "secid": "100.STI",
        "klt": "101",
        "fqt": "0",
        "lmt": "10000",
        "end": date,
        "iscca": "1",
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64",
        "ut": "f057cbcbce2a86e2866ab8877db1d059",
        "forcect": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "date",
        "-",
        "open",
        "close",
        "high",
        "low",
        "volume",
        "amount",
        "_",
        "-",
        "open",
        "close",
        "high",
        "low",
    ]
    num = temp_df["date"].index[-1] + 791
    return num


def futures_settlement_price_sgx(date: str = "20231107") -> pd.DataFrame:
    """
    新加坡交易所-衍生品-历史数据-历史结算价格
    https://www.sgx.com/zh-hans/research-education/derivatives
    :param date: 交易日
    :type date: str
    :return: 所有期货品种的在指定交易日的历史结算价格
    :rtype: pandas.DataFrame
    """
    num = __fetch_ftse_index_futu(date)
    url = f"https://links.sgx.com/1.0.0/derivatives-daily/{num}/FUTURE.zip"
    r = requests.get(url)
    with zipfile.ZipFile(BytesIO(r.content)) as file:
        with file.open(file.namelist()[0]) as my_file:
            data = my_file.read().decode()
            if file.namelist()[0].endswith("txt"):
                data_df = pd.read_table(StringIO(data))
            else:
                data_df = pd.read_csv(StringIO(data))
    return data_df


if __name__ == "__main__":
    futures_settlement_price_sgx_df = futures_settlement_price_sgx(date="20240110")
    print(futures_settlement_price_sgx_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/12/12 16:30
Desc: 基金评级
https://fund.eastmoney.com/data/fundrating.html
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup


def fund_rating_all() -> pd.DataFrame:
    """
    天天基金网-基金评级-基金评级总汇
    https://fund.eastmoney.com/data/fundrating.html
    :return: 基金评级总汇
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/fundrating.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = soup.find(name="div", attrs={"id": "fundinfo"}).find("script").string
    data_content = [
        item.split("|")
        for item in data_text.split("var")[6]
        .split("=")[1]
        .strip()
        .strip(";")
        .strip('"')
        .strip("|")
        .split("|_")
    ]
    temp_df = pd.DataFrame(data_content)
    temp_df.columns = [
        "代码",
        "简称",
        "类型",
        "基金经理",
        "-",
        "基金公司",
        "-",
        "5星评级家数",
        "-",
        "-",
        "招商证券",
        "-",
        "上海证券",
        "-",
        "晨星评级",
        "-",
        "济安金信",
        "-",
        "手续费",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "代码",
            "简称",
            "基金经理",
            "基金公司",
            "5星评级家数",
            "上海证券",
            "招商证券",
            "济安金信",
            "晨星评级",
            "手续费",
            "类型",
        ]
    ]
    temp_df["5星评级家数"] = pd.to_numeric(temp_df["5星评级家数"], errors="coerce")
    temp_df["上海证券"] = pd.to_numeric(temp_df["上海证券"], errors="coerce")
    temp_df["招商证券"] = pd.to_numeric(temp_df["招商证券"], errors="coerce")
    temp_df["济安金信"] = pd.to_numeric(temp_df["济安金信"], errors="coerce")
    temp_df["晨星评级"] = pd.to_numeric(temp_df["晨星评级"], errors="coerce")
    temp_df["手续费"] = (
        pd.to_numeric(temp_df["手续费"].str.strip("%"), errors="coerce") / 100
    )
    return temp_df


def fund_rating_sh(date: str = "20230630") -> pd.DataFrame:
    """
    天天基金网-基金评级-上海证券评级
    https://fund.eastmoney.com/data/fundrating_3.html
    :param date: 日期; https://fund.eastmoney.com/data/fundrating_3.html 获取查询日期
    :type date: str
    :return: 上海证券评级
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/fundrating_3.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    date_list = [
        item["value"] for item in soup.find("select", attrs={"id": "rqoptions"})
    ]
    date_format = "-".join([date[:4], date[4:6], date[6:]])
    if date_format not in date_list:
        raise "请访问 https://fund.eastmoney.com/data/fundrating_3.html 获取查询日期"
    url = f"https://fund.eastmoney.com/data/fundrating_3_{'-'.join([date[:4], date[4:6], date[6:]])}.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    data_text = soup.find("div", attrs={"id": "fundinfo"}).find("script").string
    data_content = [
        item.split("|")
        for item in data_text.split("var")[1]
        .split("=")[1]
        .strip()
        .strip(";")
        .strip('"')
        .strip("|")
        .split("|_")
    ]
    temp_df = pd.DataFrame(data_content)
    temp_df.columns = [
        "代码",
        "简称",
        "类型",
        "基金经理",
        "_",
        "基金公司",
        "_",
        "3年期评级-3年评级",
        "3年期评级-较上期",
        "5年期评级-5年评级",
        "5年期评级-较上期",
        "单位净值",
        "日期",
        "日增长率",
        "近1年涨幅",
        "近3年涨幅",
        "近5年涨幅",
        "手续费",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "代码",
            "简称",
            "基金经理",
            "基金公司",
            "3年期评级-3年评级",
            "3年期评级-较上期",
            "5年期评级-5年评级",
            "5年期评级-较上期",
            "单位净值",
            "日期",
            "日增长率",
            "近1年涨幅",
            "近3年涨幅",
            "近5年涨幅",
            "手续费",
            "类型",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["3年期评级-3年评级"] = pd.to_numeric(
        temp_df["3年期评级-3年评级"], errors="coerce"
    )
    temp_df["3年期评级-较上期"] = pd.to_numeric(
        temp_df["3年期评级-较上期"], errors="coerce"
    )
    temp_df["5年期评级-5年评级"] = pd.to_numeric(
        temp_df["5年期评级-5年评级"], errors="coerce"
    )
    temp_df["5年期评级-较上期"] = pd.to_numeric(
        temp_df["5年期评级-较上期"], errors="coerce"
    )
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    temp_df["近1年涨幅"] = pd.to_numeric(temp_df["近1年涨幅"], errors="coerce")
    temp_df["近3年涨幅"] = pd.to_numeric(temp_df["近3年涨幅"], errors="coerce")
    temp_df["近5年涨幅"] = pd.to_numeric(temp_df["近5年涨幅"], errors="coerce")
    return temp_df


def fund_rating_zs(date: str = "20230331") -> pd.DataFrame:
    """
    天天基金网-基金评级-招商证券评级
    https://fund.eastmoney.com/data/fundrating_2.html
    :param date: 日期; https://fund.eastmoney.com/data/fundrating_2.html 获取查询日期
    :type date: str
    :return: 招商证券评级-混合型
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/fundrating_2.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    date_list = [
        item["value"] for item in soup.find("select", attrs={"id": "rqoptions"})
    ]
    date_format = "-".join([date[:4], date[4:6], date[6:]])
    if date_format not in date_list:
        raise "请访问 https://fund.eastmoney.com/data/fundrating_2.html 获取查询日期"
    url = f"https://fund.eastmoney.com/data/fundrating_2_{'-'.join([date[:4], date[4:6], date[6:]])}.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    data_text = soup.find("div", attrs={"id": "fundinfo"}).find("script").string
    data_content = [
        item.split("|")
        for item in data_text.split("var")[1]
        .split("=")[1]
        .strip()
        .strip(";")
        .strip('"')
        .strip("|")
        .split("|_")
    ]
    temp_df = pd.DataFrame(data_content)
    temp_df.columns = [
        "代码",
        "简称",
        "_",
        "基金经理",
        "_",
        "基金公司",
        "_",
        "3年期评级-3年评级",
        "3年期评级-较上期",
        "单位净值",
        "日期",
        "日增长率",
        "近1年涨幅",
        "近3年涨幅",
        "近5年涨幅",
        "手续费",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "代码",
            "简称",
            "基金经理",
            "基金公司",
            "3年期评级-3年评级",
            "3年期评级-较上期",
            "单位净值",
            "日期",
            "日增长率",
            "近1年涨幅",
            "近3年涨幅",
            "近5年涨幅",
            "手续费",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["3年期评级-3年评级"] = pd.to_numeric(
        temp_df["3年期评级-3年评级"], errors="coerce"
    )
    temp_df["3年期评级-较上期"] = pd.to_numeric(
        temp_df["3年期评级-较上期"], errors="coerce"
    )
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    temp_df["近1年涨幅"] = pd.to_numeric(temp_df["近1年涨幅"], errors="coerce")
    temp_df["近3年涨幅"] = pd.to_numeric(temp_df["近3年涨幅"], errors="coerce")
    temp_df["近5年涨幅"] = pd.to_numeric(temp_df["近5年涨幅"], errors="coerce")
    return temp_df


def fund_rating_ja(date: str = "20230331") -> pd.DataFrame:
    """
    天天基金网-基金评级-济安金信评级
    https://fund.eastmoney.com/data/fundrating_4.html
    :param date: 日期; https://fund.eastmoney.com/data/fundrating_4.html 获取查询日期
    :type date: str
    :return: 济安金信评级
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/fundrating_4.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    date_list = [
        item["value"] for item in soup.find("select", attrs={"id": "rqoptions"})
    ]
    date_format = "-".join([date[:4], date[4:6], date[6:]])
    if date_format not in date_list:
        raise "请访问 http://fund.eastmoney.com/data/fundrating_4.html 获取查询日期"
    url = f"https://fund.eastmoney.com/data/fundrating_4_{'-'.join([date[:4], date[4:6], date[6:]])}.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    data_text = soup.find("div", attrs={"id": "fundinfo"}).find("script").string
    data_content = [
        item.split("|")
        for item in data_text.split("var")[1]
        .split("=")[1]
        .strip()
        .strip(";")
        .strip('"')
        .strip("|")
        .split("|_")
    ]
    temp_df = pd.DataFrame(data_content)
    temp_df.columns = [
        "代码",
        "简称",
        "类型",
        "基金经理",
        "_",
        "基金公司",
        "_",
        "3年期评级-3年评级",
        "3年期评级-较上期",
        "单位净值",
        "日期",
        "日增长率",
        "近1年涨幅",
        "近3年涨幅",
        "近5年涨幅",
        "手续费",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "代码",
            "简称",
            "基金经理",
            "基金公司",
            "3年期评级-3年评级",
            "3年期评级-较上期",
            "单位净值",
            "日期",
            "日增长率",
            "近1年涨幅",
            "近3年涨幅",
            "近5年涨幅",
            "手续费",
            "类型",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["3年期评级-3年评级"] = pd.to_numeric(
        temp_df["3年期评级-3年评级"], errors="coerce"
    )
    temp_df["3年期评级-较上期"] = pd.to_numeric(
        temp_df["3年期评级-较上期"], errors="coerce"
    )
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    temp_df["近1年涨幅"] = pd.to_numeric(temp_df["近1年涨幅"], errors="coerce")
    temp_df["近3年涨幅"] = pd.to_numeric(temp_df["近3年涨幅"], errors="coerce")
    temp_df["近5年涨幅"] = pd.to_numeric(temp_df["近5年涨幅"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fund_rating_all_df = fund_rating_all()
    print(fund_rating_all_df)

    fund_rating_sh_df = fund_rating_sh(date="20230630")
    print(fund_rating_sh_df)

    fund_rating_zs_df = fund_rating_zs(date="20230331")
    print(fund_rating_zs_df)

    fund_rating_ja_df = fund_rating_ja(date="20230331")
    print(fund_rating_ja_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/11/15 20:40
Desc: 艺恩
视频放映
电视剧集
综艺节目
https://www.endata.com.cn/Video/index.html
"""

import json
import os

import pandas as pd
import requests
import py_mini_racer


def _get_js_path(name: str = "", module_file: str = "") -> str:
    """
    get JS file path
    :param name: file name
    :type name: str
    :param module_file: filename
    :type module_file: str
    :return: 路径
    :rtype: str
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "movie", name)
    return module_json_path


def _get_file_content(file_name: str = "jm.js"):
    """
    read the file content
    :param file_name: filename
    :type file_name: str
    :return: file content
    :rtype: str
    """
    setting_file_name = file_name
    setting_file_path = _get_js_path(setting_file_name, __file__)
    with open(setting_file_path) as f:
        file_data = f.read()
    return file_data


def decrypt(origin_data: str = "") -> str:
    """
    解密艺恩的加密数据
    :param origin_data: 解密前的字符串
    :type origin_data: str
    :return: 解密后的字符串
    :rtype: str
    """
    file_data = _get_file_content(file_name="jm.js")
    ctx = py_mini_racer.MiniRacer()
    ctx.eval(file_data)
    data = ctx.call("webInstace.shell", origin_data)
    return data


def video_tv() -> pd.DataFrame:
    """
    艺恩-视频放映-电视剧集
    https://www.endata.com.cn/Video/index.html
    :return: 电视剧集
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {"tvType": 2, "MethodName": "BoxOffice_GetTvData_PlayIndexRank"}
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    report_date = data_json["Data"]["Table1"][0]["MaxDate"]
    temp_df.columns = [
        "排序",
        "名称",
        "类型",
        "播映指数",
        "用户热度",
        "媒体热度",
        "观看度",
        "好评度",
    ]
    temp_df = temp_df[
        ["排序", "名称", "类型", "播映指数", "媒体热度", "用户热度", "好评度", "观看度"]
    ]
    temp_df["统计日期"] = report_date
    return temp_df


def video_variety_show() -> pd.DataFrame:
    """
    艺恩-视频放映-综艺节目
    https://www.endata.com.cn/Video/index.html
    :return: 综艺节目
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {"tvType": 8, "MethodName": "BoxOffice_GetTvData_PlayIndexRank"}
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    report_date = data_json["Data"]["Table1"][0]["MaxDate"]
    temp_df.columns = [
        "排序",
        "名称",
        "类型",
        "播映指数",
        "用户热度",
        "媒体热度",
        "观看度",
        "好评度",
    ]
    temp_df = temp_df[
        ["排序", "名称", "类型", "播映指数", "媒体热度", "用户热度", "好评度", "观看度"]
    ]
    temp_df["统计日期"] = report_date
    return temp_df


if __name__ == "__main__":
    video_tv_df = video_tv()
    print(video_tv_df)

    video_variety_show_df = video_variety_show()
    print(video_variety_show_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/15 22:30
Desc: 港股股票指数数据-新浪-东财
所有指数-实时行情数据和历史行情数据
https://finance.sina.com.cn/realstock/company/sz399552/nc.shtml
https://quote.eastmoney.com/gb/zsHSTECF2L.html
"""

import re

import pandas as pd
import requests
import py_mini_racer

from functools import lru_cache

from akshare.stock.cons import hk_js_decode
from akshare.utils.func import fetch_paginated_data


def _replace_comma(x) -> str:
    """
    去除单元格中的 ","
    :param x: 单元格元素
    :type x: str
    :return: 处理后的值或原值
    :rtype: str
    """
    if "," in str(x):
        return str(x).replace(",", "")
    else:
        return x


def get_hk_index_page_count() -> int:
    """
    指数的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#zs_hk
    :return: 需要抓取的指数的总页数
    :rtype: int
    """
    res = requests.get(
        "https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getNameCount?node=zs_hk"
    )
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def stock_hk_index_spot_sina() -> pd.DataFrame:
    """
    新浪财经-行情中心-港股指数
    大量采集会被目标网站服务器封禁 IP, 如果被封禁 IP, 请 10 分钟后再试
    https://vip.stock.finance.sina.com.cn/mkt/#zs_hk
    :return: 所有指数的实时行情数据
    :rtype: pandas.DataFrame
    """
    url = (
        "https://hq.sinajs.cn/rn=mtf2t&list=hkCES100,hkCES120,hkCES280,hkCES300,hkCESA80,hkCESG10,"
        "hkCESHKM,hkCSCMC,hkCSHK100,hkCSHKDIV,hkCSHKLC,hkCSHKLRE,hkCSHKMCS,hkCSHKME,hkCSHKPE,hkCSHKSE,"
        "hkCSI300,hkCSRHK50,hkGEM,hkHKL,hkHSCCI,hkHSCEI,hkHSI,hkHSMBI,hkHSMOGI,hkHSMPI,hkHSTECH,hkSSE180,"
        "hkSSE180GV,hkSSE380,hkSSE50,hkSSECEQT,hkSSECOMP,hkSSEDIV,hkSSEITOP,hkSSEMCAP,hkSSEMEGA,hkVHSI"
    )
    headers = {"Referer": "https://vip.stock.finance.sina.com.cn/"}
    r = requests.get(url, headers=headers)
    data_text = r.text
    data_list = [
        item.split('"')[1].split(",")
        for item in data_text.split("\n")
        if len(item.split('"')) > 1
    ]
    temp_df = pd.DataFrame(data_list)
    temp_df.columns = [
        "代码",
        "名称",
        "今开",
        "昨收",
        "最高",
        "最低",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "昨收",
            "今开",
            "最高",
            "最低",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    return temp_df


def stock_hk_index_daily_sina(symbol: str = "CES100") -> pd.DataFrame:
    """
    新浪财经-港股指数-历史行情数据
    https://stock.finance.sina.com.cn/hkstock/quotes/CES100.html
    :param symbol: CES100, 港股指数代码
    :type symbol: str
    :return: 历史行情数据
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/stock/hkstock/{symbol}/klc_kl.js"
    params = {"d": "2023_5_01"}
    res = requests.get(url, params=params)
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", res.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    temp_df = pd.DataFrame(dict_list)
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")
    return temp_df


def stock_hk_index_spot_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-港股-指数实时行情
    https://quote.eastmoney.com/center/gridlist.html#hk_index
    :return: 指数行情
    :rtype: pandas.DataFrame
    """
    url = "https://15.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "wbp2u": "|0|0|0|web",
        "fid": "f3",
        "fs": "m:124,m:125,m:305",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,"
        "f26,f22,f33,f11,f62,f128,f136,f115,f152",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.rename(
        columns={
            "index": "序号",
            "f2": "最新价",
            "f3": "涨跌幅",
            "f4": "涨跌额",
            "f5": "成交量",
            "f6": "成交额",
            "f12": "代码",
            "f13": "内部编号",
            "f14": "名称",
            "f15": "最高",
            "f16": "最低",
            "f17": "今开",
            "f18": "昨收",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "内部编号",
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "今开",
            "最高",
            "最低",
            "昨收",
            "成交量",
            "成交额",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


@lru_cache()
def _symbol_code_dict() -> dict:
    """
    缓存 ak.stock_hk_index_spot_em() 接口中的代码与内部编号
    https://quote.eastmoney.com/center/gridlist.html#hk_index
    :return: 代码与内部编号
    :rtype: dict
    """
    __stock_hk_index_spot_em_df = stock_hk_index_spot_em()
    symbol_code_dict = dict(
        zip(
            __stock_hk_index_spot_em_df["代码"], __stock_hk_index_spot_em_df["内部编号"]
        )
    )
    return symbol_code_dict


def stock_hk_index_daily_em(symbol: str = "HSTECF2L") -> pd.DataFrame:
    """
    东方财富网-港股-股票指数数据
    https://quote.eastmoney.com/gb/zsHSTECF2L.html
    :param symbol: 港股指数代码; 可以通过 ak.stock_hk_index_spot_em() 获取
    :type symbol: str
    :return: 指数数据
    :rtype: pandas.DataFrame
    """
    symbol_code_dict = _symbol_code_dict()
    symbol_code_dict.update(
        {
            "HSAHP": "100",
        }
    )
    symbol_str = f"{symbol_code_dict[symbol]}.{symbol}"
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "secid": symbol_str,
        "klt": "101",  # 日频率
        "fqt": "1",
        "lmt": "10000",
        "end": "20500000",
        "iscca": "1",
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64",
        "ut": "f057cbcbce2a86e2866ab8877db1d059",
        "forcect": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "date",
        "open",
        "latest",
        "high",
        "low",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[["date", "open", "high", "low", "latest"]]
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["latest"] = pd.to_numeric(temp_df["latest"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_hk_index_spot_sina_df = stock_hk_index_spot_sina()
    print(stock_hk_index_spot_sina_df)

    stock_hk_index_daily_sina_df = stock_hk_index_daily_sina(symbol="CES100")
    print(stock_hk_index_daily_sina_df)

    stock_hk_index_spot_em_df = stock_hk_index_spot_em()
    print(stock_hk_index_spot_em_df)

    stock_hk_index_daily_em_df = stock_hk_index_daily_em(symbol="HSTECH")
    print(stock_hk_index_daily_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/8/20 16:13
Desc: 期货衍生指标配置文件
西本新干线-指数数据
"""

# xgx
symbol_dict = {
    "钢材指数": "65",
    "铁矿指数": "61",
    "焦炭指数": "64",
    "煤炭指数": "1002",
    "水泥指数": "1003",
    "FTZ指数": "1100",
    "钢铁行业PMI指数": "118",
    "生产指数": "119",
    "新订单指数": "120",
    "新出口订单指数": "121",
    "产成品库存指数": "122",
    "原材料库存指数": "123",
    "沪市终端线螺每周采购量监控": "74",
    "沪螺纹钢社会库存": "72",
    "国内螺纹钢社会库存量": "67",
    "国内线材社会库存量": "68",
    "国内主要城市热轧卷板库存": "69",
    "国内主要城市冷轧卷板库存": "70",
    "国内主要城市中厚板库存": "73",
    "全国主要钢材品种库存总量": "117",
    "热轧价格走势": "108",
    "冷轧价格走势": "109",
    "中板价格走势": "110",
    "型材价格走势": "111",
    "沪二级螺纹钢价格走势": "127",
    "螺纹钢主力合约收盘价格": "179",
    "铁矿石主力合约收盘价格": "180",
    "热轧板卷主力合约收盘价格": "181",
    "焦煤主力合约收盘价格": "182",
    "焦炭主力合约收盘价格": "183",
    "存款基准利率": "52",
    "贷款基准利率": "53",
    "存款准备金率": "105",
    "人民币新增贷款（亿元）": "174",
    "广义货币供应量增速（M2，%）": "175",
    "狭义货币供应量增速（M1，%)": "176",
    "上海大额银行承兑汇票(Ⅰ)": "129",
    "上海大额银行承兑汇票(Ⅱ)": "130",
    "上海大额银行承兑汇票(Ⅲ)": "131",
    "上海大额商业承兑汇票(Ⅰ)": "132",
    "上海大额商业承兑汇票(II)": "133",
    "上海大额商业承兑汇票(III)": "134",
    "重点企业粗钢日均产量（旬报）": "99",
    "重点企业钢材库存量（旬报）": "124",
    "国内月度粗钢日均产量": "159",
    "国内月度粗钢产量": "35",
    "国内月度钢材产量": "88",
    "国内月度螺纹钢产量": "40",
    "国内月度线材产量": "41",
    "国内月度热轧板卷产量": "114",
    "国内月度冷轧板卷产量": "115",
    "国内月度中厚板产量": "116",
    "国内月度生铁产量": "177",
    "国内月度焦炭产量": "37",
    "国内月度铁矿石原矿产量": "36",
    "国内月度铁矿石进口量": "42",
    "国内月度钢材出口量": "38",
    "国内月度钢材进口量": "39",
    "国内铁矿石港口存量": "43",
    "唐山地区钢坯库存量": "161",
    "印度矿港口库存": "100",
    "波罗的海干散货指数（BDI）": "77",
    "废钢价格走势": "78",
    "钢坯价格走势": "79",
    "钢材成本指数": "178",
    "铁矿石进口月度均价": "93",
    "巴西图巴朗-北仑铁矿海运价": "94",
    "西澳-北仑铁矿海运价": "95",
    "澳大利亚粉矿价格（56.5%，日照港）": "1006",
    "澳大利亚粉矿价格(61.5%青岛港，元/吨）": "106",
    "巴西粉矿价格（ 65% 日照港，元/吨）": "107",
    "62%铁矿石指数": "125",
    "63.5%印度粉矿外盘报价": "126",
    "国民生产总值季度增速（GDP）": "166",
    "居民消费物价指数（CPI）": "30",
    "工业生产者出厂价格指数（PPI）": "165",
    "制造业采购经理指数（PMI）": "104",
    "月度建筑安装工程投资额": "91",
    "月度固定资产投资额": "32",
    "月度房地产建设投资额": "34",
    "城填固定资产投资增速（累计值，%）": "171",
    "房地产开发投资增速（累计值，%）": "167",
    "土地购置面积同比增速（累计值，%）": "168",
    "房屋新开工面积同比增速（累计值，%）": "169",
    "商品房销售面积同比增速（累计值，%）": "170",
    "钢铁业固定资产投资增速（累计值，%）": "172",
    "CRU全球": "80",
    "CRU长材": "81",
    "CRU扁平材": "82",
    "CRU北美": "83",
    "CRU欧洲": "84",
    "CRU亚洲": "85",
    "全球粗钢月度产量（万吨）": "162",
    "全球粗钢日均产量（万吨）": "163",
    "全球粗钢产能利用率（%）": "164",
}

xgx_code_url = "http://www.96369.net/Other/ValidateCode.aspx"
xgx_main_url = "http://www.96369.net/indices/{}"
xgx_headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Content-Length": "57",
    "Content-Type": "application/x-www-form-urlencoded",
    "Host": "www.96369.net",
    "Origin": "http://www.96369.net",
    "Pragma": "no-cache",
    "Referer": "http://www.96369.net/indices/67",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36",
}
xgx_short_headers = {
    "Accept": "image/webp,image/apng,image/*,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "www.96369.net",
    "Pragma": "no-cache",
    "Referer": "http://www.96369.net/indices/67",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36",
}
# csa
csa_params_url = "https://www.jiaoyifamen.com/tools/nav/spread"
csa_url_spread = "https://www.jiaoyifamen.com/tools/future/spread/free"
csa_url_ratio = "https://www.jiaoyifamen.com/tools/future/valence/free"
csa_url_customize = "https://www.jiaoyifamen.com/tools/future/customize"

csa_payload = {"type1": "RB", "code1": "01", "type2": "RB", "code2": "05"}



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/3/21 12:00
Desc: 东方财富网-数据中心-现货与股票
https://data.eastmoney.com/ifdata/xhgp.html
"""

import pandas as pd
import requests

from akshare.utils import demjson


def futures_spot_stock(symbol: str = "能源") -> pd.DataFrame:
    """
    东方财富网-数据中心-现货与股票
    https://data.eastmoney.com/ifdata/xhgp.html
    :param symbol: choice of {'能源', '化工', '塑料', '纺织', '有色', '钢铁', '建材', '农副'}
    :type symbol: str
    :return: 现货与股票上下游对应数据
    :rtype: pandas.DataFrame
    """
    map_dict = {
        "能源": 0,
        "化工": 1,
        "塑料": 2,
        "纺织": 3,
        "有色": 4,
        "钢铁": 5,
        "建材": 6,
        "农副": 7,
    }
    url = "https://data.eastmoney.com/ifdata/xhgp.html"
    headers = {
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,"
        "image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "data.eastmoney.com",
        "Pragma": "no-cache",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    data_text = r.text
    temp_json = demjson.decode(
        data_text[
            data_text.find("pagedata") : data_text.find(
                "/newstatic/js/common/emdataview.js"
            )
        ]
        .strip("pagedata= ")
        .strip(';\n        </script>\n        <script src="')
    )
    date_list = list(temp_json["dates"].values())
    temp_json = temp_json["datas"]
    temp_df = temp_json[map_dict.get(symbol)]
    temp_df = pd.DataFrame(temp_df["list"])
    xyyh_list = [
        "-" if item == [] else ", ".join([inner_item["name"] for inner_item in item])
        for item in temp_df["xyyhs"].tolist()
    ]
    scs_list = [
        "-" if item == [] else ", ".join([inner_item["name"] for inner_item in item])
        for item in temp_df["scss"].tolist()
    ]
    temp_df["scss"] = scs_list
    temp_df["xyyhs"] = xyyh_list
    temp_df.columns = [
        "商品名称",
        date_list[0],
        date_list[1],
        date_list[2],
        date_list[3],
        date_list[4],
        "最新价格",
        "近半年涨跌幅",
        "生产商",
        "下游用户",
    ]
    temp_df[date_list[0]] = pd.to_numeric(temp_df[date_list[0]], errors="coerce")
    temp_df[date_list[1]] = pd.to_numeric(temp_df[date_list[1]], errors="coerce")
    temp_df[date_list[2]] = pd.to_numeric(temp_df[date_list[2]], errors="coerce")
    temp_df[date_list[3]] = pd.to_numeric(temp_df[date_list[3]], errors="coerce")
    temp_df["最新价格"] = pd.to_numeric(temp_df["最新价格"], errors="coerce")
    temp_df["近半年涨跌幅"] = pd.to_numeric(temp_df["近半年涨跌幅"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    futures_spot_stock_df = futures_spot_stock(symbol="能源")
    print(futures_spot_stock_df)

    for sector in ["能源", "化工", "塑料", "纺织", "有色", "钢铁", "建材", "农副"]:
        futures_spot_stock_df = futures_spot_stock(symbol=sector)
        print(futures_spot_stock_df)
